/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.exmachinatech.charcuterie;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.lang3.text.WordUtils;

/**
 * Hello world!
 *
 */
public class App
{
    public static void main( final String[] args ) throws Exception {

    	// TODO: consider more robust commandline arg support


    	if( 1 == args.length ){

    		if( "--version".equals( args[0] ) ){

    			showProjectInfo() ;
        		System.exit( 0 ) ;

    		}else if( "--help".equals( args[0] ) ){

    			printHelpPreamble() ;
        		System.exit( 0 ) ;

        	}else if( "--list".equals( args[0] ) ){

        		showProjectInfo() ;

        		System.out.println( "Available UDFs:" );
        		System.out.println();
        		System.out.println( "---" );

        		for( final String className : _UDFS.values() ){
        			printShortHelp( className ) ;
        			System.out.println( "\n---\n" );
        		}

        		System.exit( 0 ) ;

        	}else if( "--all-examples".equals( args[0] ) ){

        		showProjectInfo() ;

        		System.out.println( "---" );

        		for( final String className : _UDFS.values() ){
        			printExample( className ) ;
        			System.out.println( "\n---\n" );
        		}

        		System.exit( 0 ) ;

        	}else{

        		System.err.println( "Unsupported argument '" + args[0] + "'" );
        		System.exit( 1 ) ;
        	}

    	}else if( 2 == args.length ){

    		if( "--example".equals( args[0] ) ){

    			final String classOfInterestFullName =  args[1] ;
        		final String classOfInterestShortName = getClassShortName( classOfInterestFullName ) ;

        		if( _UDFS.containsKey( classOfInterestShortName ) ){
        			printExample( _UDFS.get( classOfInterestFullName ) ) ;
        		}else{
            		System.err.println( "Unable to find class matching '" + classOfInterestShortName + "'" );
            		System.exit( 1 ) ;
            	}

    		}else{

    			System.err.println( "Unsupported argument '" + args[0] + "'" );
    			printHelpPreamble() ;
        		System.exit( 1 ) ;

    		}


    	}else{

    		printHelpPreamble() ;
    		System.exit( 0 ) ;

    	}

    }

	public static Properties getDocsForUDF( final String udfClassName ) throws Exception {

		final InputStream helpStream = App.class.getResourceAsStream( "/help/en/" + getClassShortName( udfClassName ) + ".xml" ) ;

		final Properties result = new Properties() ;
		result.loadFromXML( helpStream ) ;

		return( result ) ;
	}

	public static void showProjectInfo() throws Exception {

		final ProjectInfo projectInfo = new ProjectInfo() ;

		System.out.println();

		System.out.println( "This is " + projectInfo.getProjectName() ) ;
		System.out.println( "  version " + projectInfo.getProjectVersion() ) ;
		System.out.println( "  built " + projectInfo.getProjectBuildDate() ) ;
		System.out.println( "  " + projectInfo.getProjectURL() ) ;

		System.out.println();

		System.out.println( "This version of " + projectInfo.getProjectName() + " was built using " ) ;
		System.out.println( "  Apache Hadoop version " + projectInfo.getHadoopVersion() ) ;
		System.out.println( "  Apache Pig version " + projectInfo.getPigVersion() ) ;

		System.out.println();


		return ;

	}

	public static void printHelpPreamble() throws Exception {


		final ProjectInfo projectInfo = new ProjectInfo() ;
		final String projectName = projectInfo.getProjectName() ;

		System.out.println();

		System.out.println(
			projectName +
			"  version " + projectInfo.getProjectVersion() +
			"  built " + projectInfo.getProjectBuildDate()
		) ;
		System.out.println( "  " + projectInfo.getProjectURL() ) ;

		System.out.println();


		System.out.println( "This JAR contains Pig UDFs, which are intended for us inside Pig scripts." ) ;
		System.out.println() ;

		System.out.println( "For more details: " ) ;
		System.out.println( "  java -jar [" + projectName + " JAR] --help" ) ;
		System.out.println( "      print this message" ) ;
		System.out.println();

		System.out.println( "  java -jar [" + projectName + " JAR] --list" ) ;
		System.out.println( "      list the available UDFs in this JAR" ) ;
		System.out.println();

		System.out.println( "  java -jar [" + projectName + " JAR] --example [UDF name]" ) ;
		System.out.println( "      print an example for the specified UDF" ) ;
		System.out.println();

		System.out.println( "  java -jar [" + projectName + " JAR] --all-examples" ) ;
		System.out.println( "      print examples for all UDFs" ) ;
		System.out.println();

		System.out.println( "  java -jar [" + projectName + " JAR] --version" ) ;
		System.out.println( "      get project version info" ) ;
		System.out.println();

		System.out.println() ;

	}


	public static void printShortHelp( final String udfClassName ){

		try{

			System.out.print( "\"" + udfClassName + "\"\n" ) ;

			final Properties udfDocs = getDocsForUDF( getClassShortName( udfClassName ) ) ;

			if( Boolean.parseBoolean( udfDocs.getProperty( "isExperimental" ) ) ){
				System.out.print( " (EXPERIMENTAL)" ) ;
			}

			System.out.println() ;

			System.out.println(
				WordUtils.wrap(
					udfDocs.getProperty( "description" ) ,
					CommonConstants.WORD_WRAP_LENGTH
				)
			);


		}catch( final Exception swallowed ){
			System.out.println( swallowed ) ;
		}

	}


	public static void printExample( final String udfClassName ){

		System.out.print( "\"" + udfClassName + "\"" ) ;

		try{

			final Properties udfDocs = getDocsForUDF( getClassShortName( udfClassName ) ) ;

			if( Boolean.parseBoolean( udfDocs.getProperty( "isExperimental" ) ) ){
				System.out.print( " (EXPERIMENTAL)" ) ;
			}

			System.out.println( "\n" ) ;


			System.out.println(
				WordUtils.wrap(
					udfDocs.getProperty( "description" ) ,
					CommonConstants.WORD_WRAP_LENGTH
				)
			);

			System.out.println(
					udfDocs.getProperty( "example" )
			);


		}catch( final Exception swallowed ){
			System.err.println( swallowed );
		}

	}

	private static String getClassShortName( final String fullName ){

		String result = null ;

		final String[] pieces = fullName.split( "\\." ) ;
		final int itemCount = pieces.length ;

		if( 1 == itemCount ){
			result = pieces[0] ;
		}else{
			result = pieces[ itemCount - 1 ] ;
		}

		return( result ) ;

	}

	// it would be easier to use classes, instead of a map; but
	// this project's POM specifies Pig as a provided dependency,
	// which means it's not available when we invoke this as an
	// executable JAR.
	private static Map<String,String> _UDFS ;


	static {

		_UDFS = new HashMap<String,String>() ;

		_UDFS.put( "EmailExtractor" , "net.exmachinatech.charcuterie.pig.eval.EmailExtractor" ) ;
		_UDFS.put( "HTMLExtractor" , "net.exmachinatech.charcuterie.pig.eval.HTMLExtractor" ) ;
		_UDFS.put( "NGramTermTokenizer" , "net.exmachinatech.charcuterie.pig.eval.NGramTermTokenizer" ) ;
		_UDFS.put( "SentenceCounter" , "net.exmachinatech.charcuterie.pig.eval.SentenceCounter" ) ;
		_UDFS.put( "SentenceTokenizer" , "net.exmachinatech.charcuterie.pig.eval.SentenceTokenizer" ) ;
		_UDFS.put( "StripHTML" , "net.exmachinatech.charcuterie.pig.eval.StripHTML" ) ;
		_UDFS.put( "TotalTermCount" , "net.exmachinatech.charcuterie.pig.eval.TotalTermCount" ) ;
		_UDFS.put( "TermFrequency" , "net.exmachinatech.charcuterie.pig.eval.TermFrequency" ) ;
		_UDFS.put( "TermTokenizer" , "net.exmachinatech.charcuterie.pig.eval.TermTokenizer" ) ;
		_UDFS.put( "UniqueTermCount" , "net.exmachinatech.charcuterie.pig.eval.UniqueTermCount" ) ;
		_UDFS.put( "XMLExtractorMultipleValue" , "net.exmachinatech.charcuterie.pig.eval.XMLExtractorMultipleValue" ) ;
		_UDFS.put( "XMLExtractorSingleValue" , "net.exmachinatech.charcuterie.pig.eval.XMLExtractorSingleValue" ) ;

	} ;


}
