/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package net.exmachinatech.charcuterie;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class ProjectInfo {

	public ProjectInfo() throws IOException {
		_configProperties = new Properties() ;
		final InputStream configStream = Thread.currentThread().getContextClassLoader().getResourceAsStream( _CONFIG_FILE ) ;
		_configProperties.load( configStream ) ;
	}

	private String getTrimmedProperty( final String propertyName ){
		return( _configProperties.getProperty( propertyName , "UNSET" ).trim() ) ;
	}
	public String getProjectName(){
		return( getTrimmedProperty( _PROPERTYNAME__PROJECT_NAME ) ) ;
	}

	public String getProjectVersion(){
		return( getTrimmedProperty( _PROPERTYNAME__PROJECT_VERSION ) ) ;
	}

	public String getProjectURL(){
		return( getTrimmedProperty( _PROPERTYNAME__PROJECT_URL ) ) ;
	}

	public String getProjectBuildDate(){
		return( getTrimmedProperty( _PROPERTYNAME__PROJECT_BUILD_DATE ) ) ;
	}



	public String getHadoopVersion(){
		return( getTrimmedProperty( _PROPERTYNAME__HADOOP_VERSION) ) ;
	}

	public String getPigVersion(){
		return( getTrimmedProperty( _PROPERTYNAME__PIG_VERSION ) ) ;
	}



	private static final String _CONFIG_FILE = "project-info.properties" ;

	private static final String _PROPERTYNAME__PROJECT_NAME = "project.main.name" ;
	private static final String _PROPERTYNAME__PROJECT_VERSION = "project.main.version" ;
	private static final String _PROPERTYNAME__PROJECT_URL = "project.main.url" ;
	private static final String _PROPERTYNAME__PROJECT_BUILD_DATE = "project.main.build_date" ;

	private static final String _PROPERTYNAME__HADOOP_VERSION = "dependency.hadoop.version" ;
	private static final String _PROPERTYNAME__PIG_VERSION = "dependency.pig.version" ;

	private final Properties _configProperties ;

} // ProjectInfo
