/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package net.exmachinatech.charcuterie.pig.eval;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import net.exmachinatech.charcuterie.tools.MimeTool;

import org.apache.james.mime4j.dom.Message;
import org.apache.pig.EvalFunc;
import org.apache.pig.PigWarning;
import org.apache.pig.data.DataType;
import org.apache.pig.data.Tuple;
import org.apache.pig.impl.logicalLayer.schema.Schema;


public class EmailExtractorReturnsMap
	extends EvalFunc<Map <String,String> >
{

	/**
	 * emit dates using whatever is the local default
	 */
	public EmailExtractorReturnsMap(){
		_mimeTool = new MimeTool() ;
		_dateFormatter = null ;
	}

	/**
	 * emit dates according to the provided format.  Said format
	 * must be suitable for java.text.SimpleDateFormat, as that
	 * is what we use internally.
	 */
	public EmailExtractorReturnsMap( final String pattern ){
		_mimeTool = new MimeTool() ;
		_dateFormatter = new SimpleDateFormat( pattern ) ;
	}


	/*
	 * TODO: accept list of fields-to-return as a ctor arg
	 */


	@Override
	public Map< String , String > exec( final Tuple input ) throws IOException {

		Map< String , String > result = null ;

		try{

			final Message rawMessage = _mimeTool.parseMessage( (String) input.get( 0 ) ) ;
			final Date rawDate = _mimeTool.getDate( rawMessage ) ;

			String formattedDate = null ;

			if( null == _dateFormatter ){
				formattedDate = rawDate.toString() ;
			}else{
				formattedDate = _dateFormatter.format( rawDate ) ;
			}

			result = new HashMap<String,String>() ;


			result.put( _FIELD_TO , _mimeTool.getTo( rawMessage ) ) ;
			result.put( _FIELD_FROM , _mimeTool.getTo( rawMessage ) ) ;
			result.put( _FIELD_CC , _mimeTool.getTo( rawMessage ) ) ;
			result.put( _FIELD_DATE , formattedDate ) ;
			result.put( _FIELD_SUBJECT , _mimeTool.getSubject( rawMessage ) ) ;
			result.put( _FIELD_BODY , _mimeTool.getBodyText( rawMessage ) ) ;

		}catch( final Exception swallowed ){
			warn( swallowed.getMessage() , PigWarning.UDF_WARNING_1 ) ;
		}

		return( result ) ;

	}

	// borrows heavily from examples in Pig UDF docs
	public Schema outputSchema( final Schema input ){

		 try{

			 final int fieldCount = input.size() ;

			 if( fieldCount != 1 ){
				 throw( new RuntimeException( "expected a single input parameter" ) ) ;
			 }


			 // first, check that all the input parameters are of the proper type
			 final byte inputFieldSchemaType = input.getField(0).type ;
			 if( DataType.CHARARRAY != inputFieldSchemaType ){
				 final String message = "Expected character data type; instead got " + DataType.findTypeName( inputFieldSchemaType ) ;
				 throw( new RuntimeException( message )) ;
			 }


			final Schema result = new Schema( new Schema.FieldSchema( "email" , DataType.MAP ) ) ;
			return( result ) ;

		 }catch( final Exception rethrow ){
			 throw( new RuntimeException( rethrow ) ) ;
		 }

	}

	private static final String _FIELD_TO = "to" ;
	private static final String _FIELD_FROM = "from" ;
	private static final String _FIELD_CC = "cc" ;
	private static final String _FIELD_DATE = "date" ;
	private static final String _FIELD_SUBJECT = "subject" ;
	private static final String _FIELD_BODY = "body" ;


	private final MimeTool _mimeTool ;
	private final DateFormat _dateFormatter ;

}
 // EmailExtractor
