/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.exmachinatech.charcuterie.pig.eval ;


import java.io.IOException;
import java.util.Collection;

import net.exmachinatech.charcuterie.tools.LuceneNGramTokenizerTool;
import net.exmachinatech.charcuterie.tools.TokenizerTool;

import org.apache.pig.EvalFunc;
import org.apache.pig.PigWarning;
import org.apache.pig.data.BagFactory;
import org.apache.pig.data.DataBag;
import org.apache.pig.data.DataType;
import org.apache.pig.data.Tuple;
import org.apache.pig.data.TupleFactory;
import org.apache.pig.impl.logicalLayer.schema.Schema;

/**
stem and tokenize a document

*/

public class NGramTermTokenizer
	extends EvalFunc<DataBag>
{

	// default to bigrams
	public NGramTermTokenizer( ){
		this( "2" ) ;
	}

	public NGramTermTokenizer( final String shingleSize ){
		// Pig UDFs can only accept strings as constructor args ..
		this( new LuceneNGramTokenizerTool( Integer.parseInt( shingleSize ) ) ) ;
	}

	// TODO: accept constructor arg or system property to specify
	//        Lucene vs OpenNLP tokenizer


	private NGramTermTokenizer( final TokenizerTool tokenizer ){
		_tupleFactory = TupleFactory.getInstance();
		_tagFactory = BagFactory.getInstance();
		_tokenizer = tokenizer ;
	}

	public DataBag exec( final Tuple input ) throws IOException {

        DataBag result = null ;

		try{

			result = _tagFactory.newDefaultBag();
			final String rawText = (String) input.get( 0 ) ;
			final Collection<String> tokens = _tokenizer.tokenize( rawText ) ;

		    for( final String token : tokens ){
		    	result.add( _tupleFactory.newTuple( token ) ) ;
		    }

		}catch( final Exception swallowed ) {
			warn( swallowed.getMessage() , PigWarning.UDF_WARNING_1 ) ;
		}

		return( result ) ;

	}

	// borrows heavily from examples in Pig UDF docs
	public Schema outputSchema( final Schema input ){

		if( input.size() != 1 ){
			throw( new RuntimeException( "This UDF requires just a single input parameter (the text to be tokenized)") ) ;
		}


		try{

			Schema bagSchema = new Schema();
			bagSchema.add(new Schema.FieldSchema("term", DataType.CHARARRAY));

			final Schema result = new Schema(
				new Schema.FieldSchema(
					getSchemaName(this.getClass().getName().toLowerCase(), input),
					bagSchema, DataType.BAG
				)
			) ;

			return( result ) ;

		}catch ( final Exception rethrow ){
			throw( new RuntimeException( rethrow ) ) ;
		}

	}

	private final TupleFactory _tupleFactory ;
	private final BagFactory _tagFactory ;
	private final TokenizerTool _tokenizer ;

} // public class NGramTermTokenizer
