/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.exmachinatech.charcuterie.pig.eval ;


import java.io.IOException;

import net.exmachinatech.charcuterie.tools.OpenNLPSentenceTokenizerTool;
import net.exmachinatech.charcuterie.tools.TokenizerTool;

import org.apache.pig.EvalFunc;
import org.apache.pig.PigWarning;
import org.apache.pig.data.DataType;
import org.apache.pig.data.Tuple;
import org.apache.pig.impl.logicalLayer.schema.Schema;

/**
count the number of sentences in a document.
*/

public class SentenceCounter
	extends EvalFunc<Integer>
{

	public SentenceCounter() throws Exception {
		_tokenizer = new OpenNLPSentenceTokenizerTool() ;
	}

	public Integer exec( final Tuple input ) throws IOException {

		Integer result = null ;

		try{

			final String documentText = (String) input.get( 0 ) ;

			result = new Integer( _tokenizer.countTerms( documentText ) ) ;

		}catch( final Exception swallowed ){
			warn( swallowed.getMessage() , PigWarning.UDF_WARNING_1 ) ;
		}

		return( result ) ;

	}

	// borrows heavily from examples in Pig UDF docs
	public Schema outputSchema( final Schema input ){

		if( input.size() != 1 ){
			throw( new RuntimeException( "This UDF requires just a single input parameter (the text to be tokenized)") ) ;
		}

		try{

			final byte inputFieldSchemaType = input.getField(0).type ;

			if( DataType.CHARARRAY != inputFieldSchemaType ){
				final String message = "Expected character type in field 0; instead got " + DataType.findTypeName( inputFieldSchemaType ) ;
				throw( new RuntimeException( message )) ;
			}

			final Schema result = new Schema( new Schema.FieldSchema( "count" , DataType.LONG ) ) ;
			return( result ) ;

		}catch( final Exception rethrow ){
			throw( new RuntimeException( rethrow ) ) ;
		}

	}


	private final TokenizerTool _tokenizer ;

} // public class SentenceCounter
