/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package net.exmachinatech.charcuterie.pig.eval;

import java.io.IOException;

import org.apache.pig.EvalFunc;
import org.apache.pig.PigWarning;
import org.apache.pig.data.DataType;
import org.apache.pig.data.Tuple;
import org.apache.pig.impl.logicalLayer.schema.Schema;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

/**
 * Strip HTML from the provided string
 */

public class StripHTML
	extends EvalFunc<String>
{

	public String exec( final Tuple input ) throws IOException {

        String result = null ;

		try{

			final String rawHTML = (String) input.get( 0 ) ;
			final Document doc = Jsoup.parse( rawHTML ) ;

			result = doc.text().trim() ;

		}catch( final Exception swallowed ){
			warn( swallowed.getMessage() , PigWarning.UDF_WARNING_1 ) ;
		}

		return( result ) ;

	}

	// borrows heavily from examples in Pig UDF docs
	public Schema outputSchema( final Schema input ){

		 if( input.size() != 1 ){
			 throw( new RuntimeException( "expected only one input parameter" ) ) ;
		 }


		 try{

			final Schema result = new Schema( new Schema.FieldSchema( null , DataType.CHARARRAY ) ) ;
			return( result ) ;

		 }catch( final Exception rethrow ){
			throw( new RuntimeException( rethrow ) ) ;
		}

	}


} // public class HTMLExtractor
