/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.exmachinatech.charcuterie.pig.eval ;


import java.io.IOException;
import java.util.ArrayList;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import net.exmachinatech.charcuterie.tools.XMLTools;

import org.apache.pig.EvalFunc;
import org.apache.pig.PigWarning;
import org.apache.pig.data.DataType;
import org.apache.pig.data.Tuple;
import org.apache.pig.data.TupleFactory;
import org.apache.pig.impl.logicalLayer.schema.Schema;
import org.apache.pig.impl.logicalLayer.schema.Schema.FieldSchema;
import org.w3c.dom.Document;

/**
pass in an XPath path to extract multiple values from an XML document.

*/

public class XMLExtractorMultipleValue
	extends EvalFunc<Tuple>
{

	public XMLExtractorMultipleValue(){
		_tupleFactory = TupleFactory.getInstance();
		_xpathExpressions = new ArrayList<String>() ;
	}

	public Tuple exec( final Tuple input ) throws IOException {


        Tuple result = null ;

		_xpathExpressions.clear() ;

		try{

			final int expressionCount = input.size() - 1 ;

			result = _tupleFactory.newTuple( expressionCount ) ;

			final String xmlText = (String) input.get( 0 ) ;

			for( int ix = 1 ; ix <= expressionCount ; ++ix ){
				_xpathExpressions.add( (String) input.get( ix ) ) ;
			}

			final Document doc = XMLTools.stringToXML( xmlText ) ;
		    final XPathFactory factory = XPathFactory.newInstance();
		    final XPath xpath = factory.newXPath();

		    for( int ix = 0 ; ix < expressionCount ; ++ix ){
			    // TODO: catch blanks returned from xpath.evaluate()?
		    	result.set( ix , xpath.evaluate( _xpathExpressions.get( ix ) , doc ).trim() ) ;
		    }


		}catch( final Exception swallowed ){
			warn( swallowed.getMessage() , PigWarning.UDF_WARNING_1 ) ;
		}

		return( result ) ;

	}

	// borrows heavily from examples in Pig UDF docs
	public Schema outputSchema( final Schema input ){

		 final int fieldCount = input.size() ;

		 if( fieldCount < 2 ){
			 throw( new RuntimeException( "expected two or more input parameters" ) ) ;
		 }

		 try{

			 Schema tupleSchema = new Schema();

			 for( int ix = 1 ; ix < fieldCount ; ++ix ){
				 tupleSchema.add( new FieldSchema( null , DataType.CHARARRAY )) ;
			 }

            final Schema result = new Schema(
        		new Schema.FieldSchema(
    				getSchemaName(this.getClass().getName().toLowerCase(), input),
    				tupleSchema, DataType.TUPLE
				)
    		) ;

			return( result ) ;

		}catch( final Exception rethrow ){
			throw( new RuntimeException( rethrow ) ) ;
		}

	}

	private final TupleFactory _tupleFactory ;
	private final ArrayList<String> _xpathExpressions ;

} // public class XMLExtractorMultipleValue
