/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.exmachinatech.charcuterie.pig.eval ;


import java.io.IOException;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import net.exmachinatech.charcuterie.tools.XMLTools;

import org.apache.pig.EvalFunc;
import org.apache.pig.PigWarning;
import org.apache.pig.data.DataType;
import org.apache.pig.data.Tuple;
import org.apache.pig.impl.logicalLayer.schema.Schema;
import org.w3c.dom.Document;

/**
pass in an XPath path to extract a single value from an XML document.

*/

public class XMLExtractorSingleValue
	extends EvalFunc<String>
{

	public String exec( final Tuple input ) throws IOException {

		String result = null ;

		try{

			final String xmlText = (String) input.get( 0 ) ;
			final String xpathExpression = (String) input.get( 1 ) ;

			final Document doc = XMLTools.stringToXML( xmlText ) ;

		    final XPathFactory factory = XPathFactory.newInstance();
		    final XPath xpath = factory.newXPath();

		    // TODO: catch blanks returned from xpath.evaluate()?

		    result = xpath.evaluate( xpathExpression , doc ).trim() ;

		}catch( final Exception swallowed ){
			warn( swallowed.getMessage() , PigWarning.UDF_WARNING_1 ) ;
		}

		return( result ) ;

	}

	// borrows heavily from examples in Pig UDF docs
	public Schema outputSchema( final Schema input ){

		 if( input.size() != 2 ){
			 throw( new RuntimeException( "this UDF requires two input parameters" ) ) ;
		 }


		 try{

			final Schema result = new Schema( new Schema.FieldSchema( "field" , DataType.CHARARRAY ) ) ;
			return( result ) ;

		 }catch( final Exception rethrow ){
			throw( new RuntimeException( rethrow ) ) ;
		}

	}

} // public class XMLExtractorSingleValue
