/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package net.exmachinatech.charcuterie.tools;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.tokenattributes.CharTermAttribute;

public class LuceneTokenizerBase implements TokenizerTool {

	@Override
	public final Collection<String> tokenize( final String input ) throws Exception {

		final ArrayList<String> result = new ArrayList<String>() ;

		final TokenStream tokens = _analyzer.tokenStream( "unused", new StringReader( input ) ) ;
		tokens.reset() ;

		while( tokens.incrementToken() ){

			// result.add( tokens.reflectAsString( false ) ) ;
			final CharTermAttribute termAttribute = tokens.getAttribute( CharTermAttribute.class ) ;
			result.add( termAttribute.toString() ) ;
			// System.out.println( "token: " + termAttribute.toString() ) ;
		}

		tokens.end() ;
		tokens.close() ;

		return( result ) ;
	}

	@Override
	public final int countTerms( final String input ) throws Exception {

		int result = 0 ;

		final TokenStream tokens = _analyzer.tokenStream( "unused" , new StringReader( input ) ) ;

		tokens.reset() ;

		while( tokens.incrementToken() ){
			++result ;
		}

		tokens.end() ;

		return( result ) ;
	}

	protected final void setAnalyzer( final Analyzer analyzer ){
		_analyzer = analyzer ;
	}

	private Analyzer _analyzer ;

} // LuceneTokenizerBase
