/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package net.exmachinatech.charcuterie.tools;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.CharBuffer;
import java.util.Date;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.commons.lang3.StringUtils;
import org.apache.james.mime4j.dom.Body;
import org.apache.james.mime4j.dom.Message;
import org.apache.james.mime4j.dom.Multipart;
import org.apache.james.mime4j.dom.SingleBody;
import org.apache.james.mime4j.dom.address.Address;
import org.apache.james.mime4j.dom.address.AddressList;
import org.apache.james.mime4j.dom.address.Mailbox;
import org.apache.james.mime4j.dom.address.MailboxList;
import org.apache.james.mime4j.message.DefaultMessageBuilder;
import org.apache.james.mime4j.stream.MimeConfig;


public class MimeTool {

	// TODO: accept max line length as a ctor param
	public MimeTool(){
		_messageBuilder = new DefaultMessageBuilder() ;
		// _messageBuilder.setFieldParser( new LenientFieldParser() ) ;
		final MimeConfig mimeConfig = new MimeConfig() ;
		mimeConfig.setMaxLineLen( -1 ) ;
		mimeConfig.setMaxHeaderLen( -1 ) ;
		_messageBuilder.setMimeEntityConfig( mimeConfig ) ;
	}

	public Message parseMessage( final InputStream messageStream ) throws Exception {

		final Message result = _messageBuilder.parseMessage( messageStream ) ;

		return( result ) ;

	}

	public Message parseMessage( final String messageContent ) throws Exception {
		return( parseMessage( new ByteArrayInputStream( messageContent.getBytes( "UTF-8" ) ) ) ) ;
	}



	public String getFrom( final Message message ){

		String result = null ;

		final MailboxList fromList = message.getFrom() ;
		if(
			( null != fromList )
				&&
			( ! fromList.isEmpty() )
		){
			final Mailbox mailbox = fromList.get(0) ;
			result = mailbox.getAddress() ;
		}

		return( result ) ;
	}

	public String getTo( final Message message ){
		return( sortAndMergeAddresses( message.getTo() )) ;
	}

	public String getCC( final Message message ){
		return( sortAndMergeAddresses( message.getCc() )) ;
	}

	// TODO: also add "getAllRecipients()" ?

	public Date getDate( final Message message ){
		return( message.getDate() ) ;
	}

	public String getSubject( final Message message ){
		return( message.getSubject() ) ;
	}


	/*
	 * naive handling of multipart messages: returns the text of the first part.
	 */

	private String getBodyText( final Body body ) throws Exception {

		if( body instanceof SingleBody ){

			final StringBuilder result = new StringBuilder() ;
			final CharBuffer buf = CharBuffer.allocate( 10240 ) ;

			final Reader reader = new InputStreamReader( ( (SingleBody) body ).getInputStream() ) ;

			int charactersRead = 0 ;
			while( ( charactersRead = reader.read( buf ) ) > 0 ){
				result.append( buf.array() , 0 , charactersRead ) ;
				buf.clear() ;
			}

			return( result.toString() ) ;

		}else if( body instanceof Multipart ){

			final Body firstPartBody = ( (Multipart) body ).getBodyParts().get(0).getBody() ;

			return( getBodyText( firstPartBody ) ) ;


		}else if( body instanceof Message ){

			throw( new IllegalArgumentException( "Not prepared to handle body type 'Message'" ) ) ;

		}else{

			throw( new IllegalArgumentException( "Unsupported message body type: " + body.getClass() ) ) ;

		}


	}

	public String getBodyText( final Message message ) throws Exception {

		return( getBodyText( message.getBody() ) ) ;

	}


	private String sortAndMergeAddresses( final AddressList toMerge ){

		String result = null ;

		if( null != toMerge ){

			final SortedSet< String > addresses = new TreeSet<String>() ;
			for( final Address address : toMerge ){
				addresses.add( address.toString() ) ;
			}

			result = StringUtils.join( addresses , "|" ) ;

		}

		return( result ) ;

	}


	private final DefaultMessageBuilder _messageBuilder ;

}
 // MimeTool
