/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package net.exmachinatech.charcuterie.tools;

import java.io.InputStream;
import java.util.Arrays;
import java.util.Collection;

import opennlp.tools.sentdetect.SentenceDetector;
import opennlp.tools.sentdetect.SentenceDetectorME;
import opennlp.tools.sentdetect.SentenceModel;

public class OpenNLPSentenceTokenizerTool implements TokenizerTool {

	public OpenNLPSentenceTokenizerTool() throws Exception {

		InputStream modelStream = null ;

		try{

			modelStream = getClass().getResourceAsStream( _OPENNLP_SENTENCE_MODEL_FILE ) ;
			// modelStream = Thread.currentThread().getContextClassLoader().getResourceAsStream( _OPENNLP_SENTENCE_MODEL_FILE ) ;

			if( null == modelStream ){
				throw( new IllegalStateException( "unable to find model file as classloader resources '" + _OPENNLP_SENTENCE_MODEL_FILE + "'" ) ) ;
			}

			_sentenceModel = new SentenceModel( modelStream ) ;
			_sentenceDetector = new SentenceDetectorME( _sentenceModel ) ;

		}catch( final Exception rethrow ){

			throw( rethrow ) ;

		}finally{

			// NOTE: based on the source code, SentenceModel does _not_ close the InputStream
			//       we pass in ... so that's up to us.

			if( null != modelStream ){
				modelStream.close() ;
			}

		}


	}

	public Collection<String> tokenize( final String toTokenize ){

		final String[] result = _sentenceDetector.sentDetect( toTokenize ) ;

		return( Arrays.asList( result ) ) ;

	}

	public int countTerms( final String input ){
		final int result = _sentenceDetector.sentDetect( input ).length ;
		return( result ) ;
	}

	private static final String _OPENNLP_SENTENCE_MODEL_FILE = "/opennlp-models/en-sent.bin";

	private final SentenceModel _sentenceModel ;
	private final SentenceDetector _sentenceDetector ;

}
 // OpenNLPSentenceTokenizerTool
