/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package net.exmachinatech.charcuterie.tools;

import java.io.InputStream;
import java.util.Arrays;
import java.util.Collection;

import opennlp.tools.tokenize.TokenizerME;
import opennlp.tools.tokenize.TokenizerModel;

public class OpenNLPTermTokenizerTool implements TokenizerTool {

	public OpenNLPTermTokenizerTool() throws Exception {

		InputStream modelStream = null ;

		try{

			modelStream = getClass().getResourceAsStream( _OPENNLP_TOKEN_MODEL_FILE ) ;
			// modelStream = Thread.currentThread().getContextClassLoader().getResourceAsStream( _OPENNLP_TOKEN_MODEL_FILE ) ;

			if( null == modelStream ){
				throw( new IllegalStateException( "unable to find model file as classloader resources '" + _OPENNLP_TOKEN_MODEL_FILE + "'" ) ) ;
			}

			_tokenizerModel = new TokenizerModel( modelStream ) ;
			_tokenDetector = new TokenizerME( _tokenizerModel ) ;

		}catch( final Exception rethrow ){

			throw( rethrow ) ;

		}finally{

			// NOTE: based on the source code, SentenceModel does _not_ close the InputStream
			//       we pass in ... so that's up to us.

			if( null != modelStream ){
				modelStream.close() ;
			}

		}


	}

	public Collection<String> tokenize( final String input ){

		final String[] tokens = _tokenDetector.tokenize( input ) ;
		final Collection<String> result = Arrays.asList( tokens ) ;

		return( result ) ;

	}

	public int countTerms( final String input ) throws Exception {
		final int result = _tokenDetector.tokenize( input ).length ;
		return( result ) ;
	}

	private static final String _OPENNLP_TOKEN_MODEL_FILE = "/opennlp-models/en-token.bin";

	private final TokenizerModel _tokenizerModel ;
	private final TokenizerME _tokenDetector ;

}
 // OpenNLPSentenceTokenizerTool
